@@ -6,12 +6,12 @@ from unittest.mock import patch, Mock, MagicMock
 import os
 import logging
 
+from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 from doorstop.core.types import Level
 from doorstop.core.document import Document
-from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 
-from doorstop.core.test import ROOT, FILES, EMPTY, NEW, MockFileObject
-from doorstop.core.test.test_item import MockItem
+from doorstop.core.test import ROOT, FILES, EMPTY, NEW
+from doorstop.core.test import MockItem, MockDocument
 
 YAML_DEFAULT = """
 settings:
@@ -36,11 +36,6 @@ settings:
 """.lstrip()
 
 
-class MockDocument(MockFileObject, Document):  # pylint: disable=W0223,R0902,R0904
-
-    """Mock Document class with stubbed file IO."""
-
-
 @patch('doorstop.settings.REORDER', False)  # pylint: disable=R0904
 @patch('doorstop.core.item.Item', MockItem)  # pylint: disable=R0904
 class TestDocument(unittest.TestCase):  # pylint: disable=R0904
@@ -103,12 +98,12 @@ class TestDocument(unittest.TestCase):  # pylint: disable=R0904
 
     @patch('doorstop.common.VERBOSITY', 2)
     def test_str(self):
-        """Verify documents can be converted to strings."""
+        """Verify a document can be converted to a string."""
         self.assertEqual("REQ", str(self.document))
 
     @patch('doorstop.common.VERBOSITY', 3)
     def test_str_verbose(self):
-        """Verify documents can be converted to strings in verbose mode."""
+        """Verify a document can be converted to a string (verbose)."""
         relpath = os.path.relpath(self.document.path, self.document.root)
         text = "REQ (@{}{})".format(os.sep, relpath)
         self.assertEqual(text, str(self.document))
@@ -117,6 +112,23 @@ class TestDocument(unittest.TestCase):  # pylint: disable=R0904
         """Verify document non-equality is correct."""
         self.assertNotEqual(self.document, None)
 
+    def test_hash(self):
+        """Verify documents can be hashed."""
+        document1 = MockDocument('path/to/fake1')
+        document2 = MockDocument('path/to/fake2')
+        document3 = MockDocument('path/to/fake2')
+        my_set = set()
+        # Act
+        my_set.add(document1)
+        my_set.add(document2)
+        my_set.add(document3)
+        # Assert
+        self.assertEqual(2, len(my_set))
+
+    def test_len(self):
+        """Verify a document has a length."""
+        self.assertEqual(5, len(self.document))
+
     def test_items(self):
         """Verify the items in a document can be accessed."""
         items = self.document.items
@@ -196,7 +208,7 @@ class TestDocument(unittest.TestCase):  # pylint: disable=R0904
     @patch('doorstop.core.document.Document.reorder')
     @patch('doorstop.core.item.Item.new')
     def test_add_item_with_level(self, mock_new, mock_reorder):
-        """Verify an item can be added to a document."""
+        """Verify an item can be added to a document with a level."""
         with patch('doorstop.settings.REORDER', True):
             item = self.document.add_item(level='4.2')
         mock_new.assert_called_once_with(None, self.document,
@@ -204,6 +216,14 @@ class TestDocument(unittest.TestCase):  # pylint: disable=R0904
                                          level='4.2')
         mock_reorder.assert_called_once_with(keep=item)
 
+    @patch('doorstop.core.item.Item.new')
+    def test_add_item_with_number(self, mock_new):
+        """Verify an item can be added to a document with a number."""
+        self.document.add_item(number=999)
+        mock_new.assert_called_once_with(None, self.document,
+                                         FILES, ROOT, 'REQ999',
+                                         level=Level('2.2'))
+
     @patch('doorstop.core.item.Item.new')
     def test_add_empty(self, mock_new):
         """Verify an item can be added to an new document."""
@@ -365,12 +385,12 @@ class TestDocument(unittest.TestCase):  # pylint: disable=R0904
         self.assertEqual(5, mock_hook.call_count)
 
     @patch('doorstop.core.item.Item.delete')
-    @patch('os.remove')
-    def test_delete(self, mock_remove, mock_delete):
+    @patch('doorstop.common.delete')
+    def test_delete(self, mock_common_delete, mock_item_delete):
         """Verify a document can be deleted."""
         self.document.delete()
-        mock_remove.assert_called_once_with(self.document.config)
-        self.assertEqual(5, mock_delete.call_count)
+        self.assertEqual(2, mock_common_delete.call_count)
+        self.assertEqual(5, mock_item_delete.call_count)
         self.document.delete()  # ensure a second delete is ignored
 
     @patch('doorstop.core.document.Document.get_issues', Mock(return_value=[]))
